function[PAS_Array,DeploymentMetData] = PUF_PAS_Effective_Volume_Modelv2(Deployment,Collection,PUF_ID,MET_ID)
%This version of the model was published by Herkert et al. XXXXXXXXXXXXXX
%based upon a model first developed by Petrich et al. (2013) Environ. Sci. Technol. 
%and Herkert et al. (2016) Environ. Sci. Technol. and describes the PUF-PAS
%sampler described therein.

%Input variable formats:
% Deployment = PUF-PAS Deployment Date formatted as YYYYMMDDHH
% Collection = PUF-PAS Collection Date formatted as YYYYMMDDHH
% PUF_ID = Numerical PUF ID for identification in output data set
% MET_ID = string of AWS site ID formatted as 'XXXXXX-XXXXX.csv' for the weather data to be used for the PUF deployment

%Specify output file name and location
    outfilename = 'Generic_Output_File_Name.csv';

%Specify the method for determining KPUF.
% KOA = Empirical relationship with temperature adjusted Koa from Shoeib and Harner (2002) Environ. Sci. Technol.
% LFER = Temperature dependent linear free energy relationship for polyurethane foam from Sprunger et al. (2007) Anal. Chem.
    KPUFmethod = 'KOA'; %KOA or LFER

%Declarations: Known variables about PUF disk parameters to be used 
%The default parameters are for PUF from Tisch Environmental used by Herkert et al. (2017) 
    diameter = 0.1397;              % Length of PUF disk (m)
    thickness = 0.0127;             % Thickness of PUF disk (m)
    dpuf = 23630;                   % Density of PUF (g/m^3)
    As = .0365;                     % Surface area of the PUF (m^2)
   
%Calculates PUF parameters based on above inputs  
    VPUF = pi*((diameter^2)/4)*thickness;   % Volume of the PUF (m^3) 

%Imports molecular weight, internal energy of octanol-air transfer, KOA, and Linear free energy parameters for all SVOC
%(This version of the code is currently formatted for 209 PCBs and Lindane)
    ChemDes = csvread('Chemical_Descriptors.csv',1,0);
    
%Import Weather Data
    [Length,T,P,WS,Qvap,DeploymentMetData] = importwdata(MET_ID,Deployment,Collection);

%Subfunction to calculate internal air velocity based on a relationship 
% from Tuduri et al. (2006) Environ. Pollut.
    Vi = InternalFlowrate(WS); %One Represent the version number
    
%Creates zero array for deployment length (Length) and all SVOCs (numSVOC)
    numSVOC = length(ChemDes(:,1));  
    PAS_Array = zeros(Length,numSVOC,5);
            
%Subfunction that calculates diffusivity in m^2/sec with T in K, P in atm, and mm in g/mol
    mm = ChemDes(:,2).';
    D = PCBDiffusivity(T,P,mm);  

%Subfunction that calculates the kinematic viscosity (m^2/s)
    Kinematic = Viscosity(T,Qvap,P);
    
%Subfunction that calculates the mass transfer coefficient (m/s)
    kv = MassTransfer(Vi,Kinematic,D,diameter,numSVOC);
    PAS_Array(:,:,1) = kv;
    
%Calculates sampling rate (Rs) in m^3/d
    Rs = kv*As*86400;
    PAS_Array(:,:,2) = Rs;
    
%Subfunction PUF/Air equilibrium partition coefficient 
    KPUF = KPUFCalculation(T,Length,ChemDes,KPUFmethod,dpuf,numSVOC);
    PAS_Array(:,:,3) = KPUF;
    
%Calculates Effective Sampling Air Volume (m^3)   
    Veff_array = VeffCalculation(Rs,VPUF,KPUF,Length,numSVOC);
    PAS_Array(:,:,4:5) = Veff_array;
    
%This is the prepanumericration of the final output data
    PAS_Final = zeros(2,numSVOC);
    PAS_Final(1,:) = PAS_Array(Length,:,5);  %Effective Air Volume (m^3)           
    PAS_Final(2,:) = mean(Rs,1);             %Average Congener specific sampling rate (m^3/d)
 
%Encode variable type numerically such that the much faster dlm write can be used and creates columns of PUF metadata for each output variable 
    rowID = zeros(2,5);
    rowID(1,:) = [PUF_ID, Deployment, Collection, Length,1];
    rowID(2,:) = [PUF_ID, Deployment, Collection, Length,2];

%Concatenates Metdata columns with final variable values 
    outData = horzcat(rowID,PAS_Final);

%Writes output to csv table based on user specifies location. '-append' means
%it will not overwrite file but add to it. ('precision', 10) ensures the
%date is outputted with the correct number of significant figures
    dlmwrite(outfilename,outData,'-append','precision',10);

end
function [Length,T,P,WS,Qvap,DeploymentMetData] = importwdata(MET_ID,Deployment,Collection)
%Subfunction that imports the weather data site specific Met_Data formatted
%to neccesary format with AWS site code to MATLAB for the PUF deployment.
        Met_Data = csvread(MET_ID); 

    %Creates index from imported Met Data of start and end time of PUF deployment
        StartTime = find(Met_Data(:,1) == Deployment);
        EndTime = find(Met_Data(:,1) == Collection);

    % Length of deployment (i.e. number of time steps for specific PUF in PAS_Array)    
        Length = EndTime-(StartTime-1); 

    % Read in Met Variables for the above time step/series
        T = Met_Data(StartTime:EndTime,2);          %Reads in Temperature (Kelvin)
        P = Met_Data(StartTime:EndTime,3);          %Reads in Pressure (Pascals)
        WS = Met_Data(StartTime:EndTime,4);         %Reads in Wind Speed (m/s)
        Qvap = Met_Data(StartTime:EndTime,6);       %Reads in Water-Vapor Mixing Ratio (kg/kg)

    % Prepares Deployment Specific Meterological Data to be outputted if desired    
        DeploymentMetData = Met_Data(StartTime:EndTime,:);
end
function Vi = InternalFlowrate(WS)
%Subfunction that calculates the internal veloctiy inside the PAS chamber (Vi)
%based on relation from from Tuduri et al. (2006) Environ. Pollut.
    ViCorrection = WS; 
    ViCorrection(ViCorrection<=1.5) = 0.1608; %set correction to 0.1608 if WS less than 1.5
    ViCorrection(ViCorrection>1.5 & ViCorrection<3.0) = 0.2385; 
    ViCorrection(ViCorrection>3.0) = 0.2703; %set correction to 0.2703 if greater than 3
    
    Vi = WS.*ViCorrection; %Creates Vi array from correction factor and WS array
end
function D = PCBDiffusivity(T,P,mm)
%Subfunction that calcuates the congener specific diffusivity for given PCB
%using temperature (T), Pressure (P), molecular weight (mm), molecular
%weight of air (mAir), and molcular weight of water (mH2O)

%Declarations: Known Variables
 VH2O = 9.5;                     % Molecular volume of water in cm^3/mol
 VAir = 20.1;                    % Molecular volume of air in cm^3/mol
 mH2O = 18.015;                  % Molar mass of water in g/mol used for diffusivity
 mAir = 28.97;                   % Molar mass of air in g/mol used for diffusivity
 
%Equation that calculates diffusivity in m^2/sec with T in K, P in atm, and m in g/mol
    D = (((10^-3*T.^1.75*((1/mAir) + (1/mH2O))^0.5)./(P.*(1/101325)*(VAir^(1/3) + VH2O^(1/3))^2))*((mm/mH2O).^-0.5))/(100^2);

end
function Kinematic = Viscosity(T,Qvap,P)
%Subfuntion that calculates kinematic viscosity (m^2/s)

%Declarations: Known Variables
Za = 0.038474;                  % Za is the pseudocritical constant for air (unitless) used for calculating the viscosity of air (calculated on the basis of air being .79 N2 and .21 O2)
Tca = 132.206;                  % Critical temperature for air (K)
Zcw = 0.231;                    % Critical compressibility factor for water (unitless) used for calculating the viscosity of water
Zw = 0.0192;                    % Pseudocritical constant for water (unitless) used for calculating the viscosity of water
Tcw = 647.4;                    % Critical temperature for water (K)
Rd = 287.058;                   % Specific gas constant for dry air in J/kg*K
Rv = 461.495;                   % Specific gas constant for water vapor J/kg*K
mH2O = 18.015;                  % Molar mass of water in g/mol used for diffusivity
mAir = 28.97;                   % Molar mass of air in g/mol used for diffusivity
 
%Calculates the viscosity of air based on temperature (T),the pseudocritical constant (Za), and the critical temperature (Tc) in K 
    MuAir = (17.78*((4.58*(T./Tca) - 1.67).^0.625)*10.^-7)./Za;

%Calculates the viscosity of water based on the temperature (T), the critical compressibility factor (Zcw), pseudocritical constant (Zw), and the critical temperature (Tc) in K
    MuWater = ((7.55*(T./Tcw)-0.55)*(Zcw.^-1.25)*10.^-7)./Zw;

%Sets a, which is an emperical factor based on Temperature in Kelvin
    a = T;
    a(a>=293.15) = 2.5; %set a to 2.5 if Temp is less than 293.15
    a(a<293.15) = 3.5; %set a to 3.5 if Temp is greater than 293.15
    
%Calculates the moles of water vapor in moles/m^3, using the density of air in kg/m^3 as (roe = P/R*T)
    Mw = Qvap.*(1/mH2O).*1000.*(P./(Rd.*T)); 

%Calculates the moles of air in moles/m^3, using the density of water as 1000 kg/m^3
    Ma = (1./Qvap).*(1/mAir).*(1000).*(1000); 

%Calculates the mole fraction of water vapor as (moles of water vapor/m^3)/((moles of water vapor/m^3)+(moles of air/m^3))
    Xw = Mw./(Mw+Ma);

%Calculates the mole fraction of air as (moles of air/m^3)/((moles of water vapor/m^3)+(moles of air/m^3))
    Xa = Ma./(Mw+Ma);

%Calculates the dynamic viscosity of moist air in kg/m-s
    DynamicMu = (Xa.*MuAir + Xw.*MuWater).*(1+((Xw-Xw.^2)./a));

%Calculates the partial pressure of water vapor air (Pv)
    Pv = Xw.*P;

%Calculates the partial pressure of dry air (Pd)
    Pd = Xa.*P;

% Calculates the density of moist air in kg/m^3
    Density = (Pd./(Rd.*T)) + (Pv./(Rv.*T));

%Calculates kinematic viscosity (m^2/s)
    Kinematic = DynamicMu./Density;
end 
function kv = MassTransfer(Vi,Kinematic,D,l,numPCB)
%Subfunction that calculates the mass transfer coefficient
    %Sets the Gamma Value --> 0.267 for ISD data or 0.315 for MERRA data   
        Gamma = 0.267; %for ISD

    %Calculates the mass transfer rate (kv) equation 3 from Herkert et al (2017) 
        kv = Gamma*(D/l).*((Vi*l./Kinematic)*ones(1,numPCB)).^(1/2).*(Kinematic*ones(1,numPCB)./D).^(1/3);  
end
function KPUF = KPUFCalculation(T,Length,ChemDes,KPUFmethod,dpuf,NumSVOC)
%Subfunction that calculates PUF/Air equilibrium partition coefficient for all SVOCs
    if isequal(KPUFmethod,'KOA') 
        %Set dU (Henry's Law), and KOA(octanol-air partition coefficient) from Chemical Descriptors csv file
            dU = ChemDes(:,3).';
            LKOA = ChemDes(:,4).';

        %Declarations: Known Variables
            Rg = 8.3144;         % Gas constant in J/mol*K

        %Adjust LKOA by temperature
            LKOAi = ones(Length,1)*LKOA-((1./T)-(1/298.15))*(dU/(2.303*Rg));

        %Calculates PUF/Air equilibrium partition coefficient for a specific PCB congener 
        %based on relationship from Shoeib and Harner (2002) Environ. Sci. Technol.
            KPUF = dpuf*(10.^(0.6366*LKOAi - 3.1774));
    end
    if isequal(KPUFmethod,'LFER') 
        %Temperature-Independent Equation Coefficients for the Abraham Model 
        %for Correlating the Gas-to-Polyurethane Foam Adsorption Coefficients
        %as given in Sprunger et al. (2007) Anal. Chem.
            cs = -1.279;
            ch = -354.607;
            es = 0.449;
            eh = 179.41;
            ss = -0.745;
            sh = -692.187;
            as = -2.541;
            ah = -1683.56;
            bs = -0.118;
            bh = -33.83;
            ls = -0.456;
            lh = -365.896;

        %Compound Specific Abraham equation descriptors. For this study all 209 PCB
        %congeners are given as presented in Abraham and Al-Hussaini (2005) J. Environ. Monit.
            E = ChemDes(:,5).';
            S = ChemDes(:,6).';
            A = ChemDes(:,7).';
            B = ChemDes(:,8).';
            L = ChemDes(:,10).';

        %Equation 11 from Sprunger et al. (2007) Anal. Chem
            logKPUF = ((cs-(ch./T))*ones(1,NumSVOC))+(es-(eh./T))*E + (ss-(sh./T))*S + (as-(ah./T))*A + (bs-(bh./T))*B + (ls-(lh./T))*L;

        %Calculates KPUF (m^3/g)
            KPUF = dpuf*((10.^(logKPUF))/1000000); 
    end
end
function Veff_array = VeffCalculation(Rs,VPUF,KPUF,Length,numPCB)
%Subfunction that calculates Effective Sampling Air Volume (m^3) 
    Veff_array = zeros(Length,numPCB,2); %creates a array of zeros to be populated by Veff
    for t = 1:Length %Creates a for loop over each hour of the time series
        if (t == 1)
            Veffdt = (Rs(t,:)/24)./VPUF.*(VPUF-(0./(KPUF(t,:)))); %Eqn 4 from Herkert et al. (2017)
            Veff_array(t,:,1) = Veffdt; %Time Step value of effective sampling volume
            Veff_array(t,:,2) = Veffdt; %Cummulative effective sampling volume
        else %(t > 1)
            Veffdt = (Rs(t,:)/24)./VPUF.*(VPUF-(Veff_array((t-1),:,2)./(KPUF(t,:))));%Eqn 4 from Herkert et al. (2017)
            Veff_array(t,:,1) = Veffdt; %Time Step value of effective sampling volume
            Veff_array(t,:,2) = Veffdt  + Veff_array((t-1),:,2);%Cummulative effective sampling volume
        end   
    end
end