function[Combined_Met_Data] = process_isd_metdatav2(metdataID,iYear,NumberYears)
% This function reformats the raw ISD Lite Files to the format required
% and combines them for the effective volume script
%
% The raw NOAA ISD Lite files can be obtained from
% ftp://ftp.ncdc.noaa.gov/pub/data/noaa/isd-lite
%
% A map tool can be used to identify which site ID that offer hourly
% meteorlogical data and corresponds to the spatial location of your study at
% https://gis.ncdc.noaa.gov/map/viewer/
%
% The user will input:
% the station ID as a string (metdataID) in the format 'XXXXXX-XXXXX'
% start year of the meteorological span (iYear) in format XXXX
% the number of years of the meteorological span(NumberYears)
%************************************************************************* 
%Loop to import each different year of the meteorological span and
%concatenate to the output array
    for i = 1:NumberYears        
        %Reads in each year's weather data and converts it to the necessary format
            Yearly_Met = convert_isd_metdatav2([metdataID,'-',num2str(iYear)]);
        
        %Combines each year of Met_Data for final output
            if i == 1
                Combined_Met_Data = Yearly_Met;
            else
                Combined_Met_Data = vertcat(Combined_Met_Data,Yearly_Met);
            end
        iYear = iYear + 1;
    end

%Outputs the combined file as XXXXXX-XXXXX.csv to the current workspace
    outputfileName = [metdataID,'.csv'];
    dlmwrite(outputfileName,Combined_Met_Data,'precision',10);

end
function [Met_Data] = convert_isd_metdatav2(metdataID)
%This subfunction reformats the data to the necessary file format required 
%by the effective volume model
        metdata = dlmread(metdataID); %Reads the Met Data into Matlab array
    
    %Creates expected date array to compare ISD date against
        [exdateArray,exdateConcate] = ExpectedDateArray(metdata(1,1)); 

    %Subfuntion that checks for missing dates and adds Null entry into the data set for missing values
        metdata = checkmissingdays(metdata(:,1:9),exdateArray);

    %Subfuntion that logs any large gaps for all variable.
    
        LargeGapThreshold = 12; %Threshold (in hours) for flagging consecutive missing values (i.e. large gaps)
        
        TALargeGaps = findlargegaps(metdata(:,5),LargeGapThreshold);
        TDLargeGaps = findlargegaps(metdata(:,6),LargeGapThreshold);
        PrLargeGaps = findlargegaps(metdata(:,7),LargeGapThreshold);
        WDLargeGaps = findlargegaps(metdata(:,8),LargeGapThreshold);
        WSLargeGaps = findlargegaps(metdata(:,9),LargeGapThreshold);
        
    %Subfunction that counts the null values (-9999) and assume 
    %for missing values it assumes continuity with previous measurements
        [TA,TAcount] = correctgaps(metdata(:,5));
        [TD,TDcount] = correctgaps(metdata(:,6));
        [Pr,Prcount] = correctgaps(metdata(:,7));
        [WD,WDcount] = correctgaps(metdata(:,8));
        [WS,WScount] = correctgaps(metdata(:,9));

     %Creates zero array to be populated by the formatted weather data 
        Met_Data = zeros(length(metdata),6);

        Met_Data(:,1) = exdateConcate;          %Date YYYYMMDDHH
        Met_Data(:,2) = (TA(:,1)*0.1)+273.15;   %Ambient Temperature in Kelvin, 0.1 is a conversion from the raw format
        Met_Data(:,3) = Pr(:,1)*10;             %Pressure in Pascal, 10 is a conversion from the raw format
        Met_Data(:,4) = WS(:,1)*0.1;            %Wind speed in m/s, 0.1 is a conversion from the raw format
        Met_Data(:,5) = WD(:,1);                %Wind Direction in degrees
        
        %Subfunction for Water-vapor mixing ratio (kg/kg). 0.1 and 10, respectively, is a conversion from the raw format
        Met_Data(:,6) = watervaporcalc(TD(:)*0.1,(TA(:)*0.1),(Pr(:)*10));   

    %Creates a Read_Me file of the data quality report in the current workspace 
        readme = fopen([metdataID,'_ReadMe.txt'],'wt');

        %Creates header for ReadMe file with summary for five weather parameters with information about total hours missing and percent of total hours missing.
            writeHeader(metdataID,length(metdata),TAcount,TDcount,Prcount,WDcount,WScount,readme);

        %List any large gaps in each meteorological data greater than the threshold (set on line 50, currently 12 hours)
            printLargeGaps(TALargeGaps,readme,LargeGapThreshold,'Temperature',exdateConcate);
            printLargeGaps(TDLargeGaps,readme,LargeGapThreshold,'Dew Point',exdateConcate); 
            printLargeGaps(PrLargeGaps,readme,LargeGapThreshold,'Pressure',exdateConcate); 
            printLargeGaps(WSLargeGaps,readme,LargeGapThreshold,'Wind Speed',exdateConcate); 
            printLargeGaps(WDLargeGaps,readme,LargeGapThreshold,'Wind Direction',exdateConcate); 
            
    %Prints readme file to screen for checking data quality and closes the Read_Me file
        type ([metdataID,'_ReadMe.txt'])     
        fclose('all'); 

end
function [exdateArray,exdateConcate] = ExpectedDateArray(Year) 
%This subfunction creates an array of the expected dates in a given year
%with hourly time steps
    Year = num2str(Year); %Reads in year
    StartTime = datetime([Year,'010100'],'Format','yyyyMMddHH'); %First Day
    EndTime = datetime([Year,'123123'],'Format','yyyyMMddHH'); %Last Day
    exdate = transpose(StartTime:hours(1):EndTime); %Create array with hour timesteps
    exdateArray = datevec(exdate); %Converts from datetime format to vector array
    exdateArray = exdateArray(:,1:4); %Remove minutes and seconds from array

%Creates string of date array
    exdateConcate = datestr(exdate,'yyyymmddHH'); %convert from datetime format to char array
    exdateConcate = str2num(exdateConcate); %convert char array to vector array
end
function outdata = checkmissingdays(indata,exdate)
%This subfuntion compares all dates/hours to the expected date/hours for
%any given year and inputs a null row for missing dates. 

    outdata = zeros(length(exdate),9); %create output file
    o = 0; %offset index
    
%Creates loop to test date from data set to the expected date 
    for i=1:length(exdate)
        if (i-o) > length(indata) %Ensures loop doesnt stop early
            outdata(i,:) = [exdate(i,1:4),ones(1,5)*-9999]; %insert Null Row
        elseif indata(i-o,1:4) == exdate(i,1:4) %Test if the data set matches expected date
            outdata(i,:) = indata(i-o,:); 
        else %If the dates do not match, an hour is missing from the ISD data, therefore a null row is added for the missing hour
            outdata(i,:) = [exdate(i,1:4),ones(1,5)*-9999]; %insert Null Row
            o = o + 1; %offset index is iterated for subsequent comparisons
        end
    end
end
function [largeGapsArray] = findlargegaps(variable,LargeGapThreshold)
%Subfunction that counts all missing measurements for a given variable and
%indexes all gaps greater than LargeGapThreshold
    x = ismember(variable,-9999); %find Null values
    changes = (find(diff(x) ~= 0)); %find where values change from valid to Null and vice versa  
    AllGaps = diff(changes); %find the length between all changes (null and valid)
    MissingValueGaps = AllGaps(1+(x(1)==1):2:end); %find length of all null value gaps
    largeGAPS = (find(MissingValueGaps >= LargeGapThreshold)); %find all null value gaps > than LargeGapThreshold
    largeGapsArray = [largeGAPS,zeros(length(largeGAPS),3)];%Create array of all LargeGaps
    if variable(1,1)== -9999
        offsetIndex = 1;
    else
        offsetIndex = 0;
    end
    for i = 1:length(largeGAPS) %iterate through for all large gap
        GapIndex = largeGapsArray(i,1); %get index of large gap
        GapSize = MissingValueGaps(GapIndex,1); %find length of large gap
        Start = changes((GapIndex*2-1+offsetIndex),1); %find beginning of large gap
        End = changes((GapIndex*2+offsetIndex),1); %find end of large gap
        largeGapsArray(i,2:4) = [GapSize,Start,End]; %log the length start and end of the gap
    end
    if length(find(variable == -9999)) == length(variable) %Creates large gap if all measurements are missing 
        largeGapsArray(1:4) = [1,length(variable),1,length(variable)];
    end
end
function [variable,count] = correctgaps(variable)
%Subfunction that counts all missing measurements for a given variable and
%indexes all gaps greater than LargeGapThreshold
    IndexList = (1:length(variable))'; %creates list of all index values
    MissingValues = (find(variable == -9999)); %find index of all Null values
    count = length(MissingValues); %counts total number of Null values
    IndexList(MissingValues)=[]; %removes missing values from index list
    if count ~= length(variable) %if count = variable length, all values are missing and continuity can not be assumed
        for i = 1:count %iterate through all Null values
            Missingindex = MissingValues(i,1); %get index for missing values
            [~,mindiff] = min(abs(IndexList-Missingindex)); %finds nearest valid entry to Null entry
            replacementindex = IndexList(mindiff); %finds index of nearest valid entry
            variable(Missingindex) = variable(replacementindex); %replace the null entry with the nearest valid replacement entry
        end
    end
end
function Water_Vapor = watervaporcalc(TD,TA,Pr)
%Subfunction that calculates the water-vapor mixing ratio from 
%ambient temperature, dewpoint, and pressure   
    RH = 100*(exp((17.625*TD)./(243.04+TD))./exp((17.625*TA)./(243.04+TA)));%Calculates the relative humidity 
    rho_sat = 6.112*10.^(17.67*TA./(TA+243.5)); %Calculates Saturation Density
    w_sat = 0.6219907.*rho_sat./(rho_sat + Pr); %Calculates Saturation mixing ratio
    Water_Vapor = w_sat .* RH;%Calculates Water-Vapor Mixing Ratio 
end
function writeHeader(metdataID,L,TAcount,TDcount,Prcount,WDcount,WScount,readme)
%Subfunction to creates header for ReadMe file with summary Summary for 
%five weather parameters with information about total hours missing and 
%percent of total hours missing. 
    header = ['---------Data Quality Summary for the input site ',metdataID,'---------'];
    dispTA = ['Total Missing Hours for Temperature    = ',num2str(TAcount),'(',num2str((TAcount/L*100)),'%)'];
    dispTD = ['Total Missing Hours for Dew Point      = ',num2str(TDcount),'(',num2str((TDcount/L*100)),'%)'];
    dispPr = ['Total Missing Hours for Pressure       = ',num2str(Prcount),'(',num2str((Prcount/L*100)),'%)'];
    dispWD = ['Total Missing Hours for Wind Direction = ',num2str(WDcount),'(',num2str((WDcount/L*100)),'%)'];
    dispWS = ['Total Missing Hours for Wind Speed     = ',num2str(WScount),'(',num2str((WScount/L*100)),'%)'];

    fprintf(readme,'%s\n\n %s\n %s\n %s\n %s\n %s\n',header,dispTA,dispTD,dispPr,dispWD,dispWS);
end
function printLargeGaps(LargeGaps,readme,LargeGapThreshold,variable,date)    
%Subfunction that list any large gaps in temperature greater than the threshold (set on line 50, currently 12 hours)
    if LargeGaps ~= 0 
        fprintf(readme,'\n---------Gaps > %d hours in Data for %s---------\n',LargeGapThreshold,variable);
        for j = 1:length(LargeGaps(:,1)) %Creates loop for all large gaps 
            Start = num2str(date(LargeGaps(j,3),1)); %Looks up date of the start of the large gap
            End = num2str(date(LargeGaps(j,4),1)); %Looks up date of the end of the large gap
            fprintf(readme,'Missing Data for %s hours between %s and %s\n',num2str(LargeGaps(j,2)),Start,End); %Prints gap to ReadMe file
        end
    end
end